# Generated by Django 2.2.10 on 2021-01-06 16:02

from django.db import migrations, models

import drc_cmis.validators


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0013_auto_20201224_1302"),
    ]

    operations = [
        migrations.AlterField(
            model_name="urlmapping",
            name="long_pattern",
            field=models.CharField(
                help_text="Part of the URL to replace. Example: https://somedomain.nl/somesubpath",
                max_length=1000,
                unique=True,
                validators=[drc_cmis.validators.url_mapping_validator],
                verbose_name="Long pattern",
            ),
        ),
        migrations.AlterField(
            model_name="urlmapping",
            name="short_pattern",
            field=models.CharField(
                help_text="Replacement string to make the URL shorter. Example: https://sd.nl",
                max_length=15,
                unique=True,
                validators=[drc_cmis.validators.url_mapping_validator],
                verbose_name="Short pattern",
            ),
        ),
    ]
