# Generated by Django 2.2.10 on 2020-07-23 13:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0004_auto_20200616_0919"),
    ]

    operations = [
        migrations.RenameField(
            model_name="cmisconfig",
            old_name="base_folder",
            new_name="base_folder_name",
        ),
    ]
