# Generated by Django 2.2.10 on 2020-07-29 05:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0005_auto_20200723_1352"),
    ]

    operations = [
        migrations.AddField(
            model_name="cmisconfig",
            name="binding",
            field=models.CharField(
                choices=[
                    ("BROWSER", "Browser binding (CMIS 1.1)"),
                    ("WEBSERVICE", "Web service binding (CMIS 1.0)"),
                ],
                default="BROWSER",
                max_length=200,
            ),
        ),
    ]
