{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}

{% block inline_field_sets %}
    {% if adminform.form.cmis_url_mapping_enabled %}
        {% for inline_admin_formset in inline_admin_formsets %}
            {% include inline_admin_formset.opts.template %}
        {% endfor %}
    {%  endif %}
{% endblock %}
