# Generated by Django 2.2.17 on 2021-02-04 10:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0013_auto_20201216_1046"),
        ("drc_cmis", "0014_auto_20210106_1602"),
    ]

    operations = []
